{% load speech_utils %}

<li class="section-signpost section-signpost--as-search-result">
<ul class="breadcrumbs">
    {% for n in section.get_ancestors %}
        {% if n.id != section.id %}
        <li><a href="{% url 'speeches:section-view' n.get_path %}">{{ n.title }}</a> </li>
        {% endif %}
    {% endfor %}
</ul>
    <a href="{{ section.get_absolute_url }}">{{ highlight|striptags_highlight }}</a>

</li>
