{% load staticfiles i18n %}<!DOCTYPE html>
<html>
    <head>
        <meta charset="utf-8">

        {% block opengraph %}
          <meta property="og:url" content="{% block og_url %}{{ request.build_absolute_uri }}{% endblock %}">
          <meta property="og:title" content="{% block og_title %}{% if title %}{{ title }} :: {% endif %}{% trans "SayIt" %}{% endblock %}">
          <meta property="og:site_name" content="{% block og_site_name %}{% trans "SayIt" %}{% endblock %}">
          <meta property="og:description" content="{% block og_description %}{% trans "Transcripts for the modern internet" %}{% endblock %}">
          <meta property="og:type" content="{% block og_type %}website{% endblock %}">
          <meta property="og:image" content="{% block og_image %}http{% if request.is_secure %}s{% endif %}://{{ request.get_host }}{% static "speeches/img/apple-touch-icon-152x152.png" %}{% endblock %}">

          {% block og_image_details %}
            <meta property="og:image:type" content="{% block og_image_type %}image/png{% endblock %}">
            {% block og_image_dimensions %}
              <meta property="og:image:width" content="{% block og_image_width %}152{% endblock %}">
              <meta property="og:image:height" content="{% block og_image_height %}152{% endblock %}">
            {% endblock og_image_dimensions %}
          {% endblock og_image_details %}
        {% endblock opengraph %}

        <title>
          {% block fulltitle %}{% block title %}{{ title }}{% endblock %}
            :: {% trans "SayIt" %}
          {% endblock %}
        </title>

        {% block extra_headers %}{% endblock %}

        {% block extra_css %}
            <link href="{% static "speeches/css/speeches.css" %}" rel="stylesheet" type="text/css">
        {% endblock extra_css %}

        <script type="text/javascript" src="{% static "speeches/js/jquery.js" %}" charset="utf-8"></script>
        <script type="text/javascript" src="{% static "speeches/js/select2-override.js" %}" charset="utf-8"></script>
        {% block extra_js_head %}{% endblock %}
    </head>

    <body id="top">

        <header>
          <div class="full-page__row">
            <div class="full-page__unit">
              <h1><a href="{% url 'speeches:home' %}">{% trans "SayIt" %}</a></h1>

              <ul class="inline-list left">
                <li><a href="{% url 'speeches:home' %}">{% trans 'Home' %}</a></li>
                <li><a href="{% url 'speeches:speaker-list' %}">{% trans 'Speakers' %}</a></li>
                <li><a href="{% url 'speeches:parentless-list' %}">{% trans 'Speeches' %}</a></li>
              </ul>

              <ul class="unstyled-list right">
                {% if request.is_user_instance %}
                  <li>{% trans 'Welcome,' %} {% firstof user.get_short_name user.get_username %}.
                      <a href="{% url 'logout' %}">{% trans 'Log out' %}</a>
                  </li>
                {% else %}
                  <li><a href="{% url 'login' %}">{% trans 'Log in' %}</a></li>
                {% endif %}
              </ul>

            </div>
          </div>
        </header>

        <div class="full-page">
          <div class="full-page__row">
            <div class="full-page__unit">
              {% for message in messages %}
                <div class="alert alert-{{ message.tags }}">
                  {{ message }}
                </div>
              {% endfor %}

              {% block content %}{% endblock %}

            </div>
          </div>
        </div>

        <script type="text/javascript" src="{% static "speeches/js/foundation/foundation.js" %}" charset="utf-8"></script>
        <script type="text/javascript" src="{% static "speeches/js/foundation/foundation.dropdown.js" %}" charset="utf-8"></script>
        <script type="text/javascript" src="{% static "speeches/js/speeches.js" %}" charset="utf-8"></script>
        {% block extra_js %}{% endblock %}
        <script>
          if (typeof $ != 'undefined' && typeof $(document).foundation != 'undefined') {
            $(document).foundation();
          }
        </script>

    </body>
</html>
