{% extends 'speeches/base.html' %}
{% load staticfiles %}

{% block extra_css %}
{{ block.super }}
<link href="{% static "speeches/mediaelement/mediaelementplayer.min.css" %}" rel="stylesheet">
{% endblock %}

{% block extra_js_head %}
{{ block.super }}
<script type="text/javascript" src="{% static "speeches/mediaelement/mediaelement-and-player.js" %}" charset="utf-8"></script>
{% endblock %}
