{% extends 'speeches/base.html' %}
{% load i18n %}

{% block title %}
{% if object %}
    {% trans "Edit speaker" %}
{% else %}
    {% trans "Add a speaker" %}
{% endif %}
{% endblock %}

{% block content %}

<div class="page-header">
  {% if object %}
    <h1>{% trans "Edit speaker" %}</h1>
  {% else %}
    <h1>{% trans "Add a speaker" %}</h1>
  {% endif %}
</div>

<form id="speaker-form" class="form-horizontal" method="post">
    {% if object %}
        {% trans "Update speaker" as submit %}
    {% else %}
        {% trans "Add speaker" as submit %}
    {% endif %}
    {% include 'speeches/form.html' with type="speaker" submit=submit %}
</form>
{% endblock %}
