


//
// Foundation Variables
//

// The default font-size is set to 100% of the browser style sheet (usually 16px)
// for compatibility with browser-based text zoom or user-set defaults.
$base-font-size: 100% !default;

// $base-line-height is 24px while $base-font-size is 16px
// $base-line-height: 150%;

// This is the default html and body font-size for the base em value.

// Since the typical default browser font-size is 16px, that makes the calculation for grid size.
// If you want your base font-size to be a different size and not have it effect grid size too,
// set the value of $em-base to $base-font-size ($em-base: $base-font-size;)
$em-base: 16 !default;

// It strips the unit of measure and returns it
@function strip-unit($num) {
  @return $num / ($num * 0 + 1);
}

// Converts "px" to "em" using the ($)em-base
@function convert-to-em($value, $base-value: $em-base)  {
  $value: strip-unit($value) / strip-unit($base-value) * 1em;
  @if ($value == 0em) { $value: 0; } // Turn 0em into 0
  @return $value;
}

// Working in ems is annoying. Think in pixels by using this handy function, emCalc(#)
// Just enter the number, no need to mention "px"
@function emCalc($values...) {
  $max: length($values); // Get the total number of parameters passed

  // If there is only 1 parameter, then return it as an integer.
  // This is done because a list can't be multiplied or divided even if it contains a single value
  @if $max == 1 { @return convert-to-em(nth($values, 1)); }

  $emValues: (); // This will eventually store the converted $values in a list
  @for $i from 1 through $max {
    $emValues: append($emValues, convert-to-em(nth($values, $i)));
  }
  @return $emValues;
}

// Change whether or not you include browser prefixes
// $experimental: true;

// Various global styles

$default-float: left;

$body-bg: #fff;
$body-font-color: #222;
$body-font-family: "Source Sans Pro", "Helvetica Neue", Arial, "Helvetica", Helvetica, sans-serif;
// $body-font-weight: normal;
// $body-font-style: normal;

// Font-smoothing

// $font-smoothing: antialiased;

// Text direction settings

// $text-direction: ltr;

// Colors

 $primary-color: #e14a55;
// $secondary-color: #e9e9e9;
 $alert-color: #f08a24;
// $success-color: #5da423;

// Make sure border radius matches unless we want it different.

$global-radius: 3px;

// $global-rounded: 1000px;

// Inset shadow shiny edges and depressions.

// $shiny-edge-size: 0 1px 0;
// $shiny-edge-color: rgba(#fff, .5);
// $shiny-edge-active-color: rgba(#000, .2);

// Control whether or not CSS classes come through in the CSS files.

// $include-html-classes: true;
// $include-print-styles: true;
// $include-html-global-classes: $include-html-classes;
// $include-html-inline-list-classes: $include-html-classes;
// $include-html-type-classes: $include-html-classes;
// $include-html-grid-classes: $include-html-classes;
// $include-html-visibility-classes: $include-html-classes;
// $include-html-button-classes: $include-html-classes;
// $include-html-form-classes: $include-html-classes;
// $include-html-custom-form-classes: $include-html-classes;
// $include-html-media-classes: $include-html-classes;
// $include-html-section-classes: $include-html-classes;
// $include-html-orbit-classes: $include-html-classes;
// $include-html-reveal-classes: $include-html-classes;
// $include-html-joyride-classes: $include-html-classes;
// $include-html-clearing-classes: $include-html-classes;
// $include-html-alert-classes: $include-html-classes;
// $include-html-nav-classes: $include-html-classes;
// $include-html-top-bar-classes: $include-html-classes;
// $include-html-label-classes: $include-html-classes;
// $include-html-panel-classes: $include-html-classes;
// $include-html-pricing-classes: $include-html-classes;
// $include-html-progress-classes: $include-html-classes;
// $include-html-magellan-classes: $include-html-classes;
// $include-html-tooltip-classes: $include-html-classes;

// Media Queries

$small-screen: 30em;
$medium-screen: 40em;
$large-screen: 60em;

// $screen: "only screen";
// $small: "only screen and (min-width: #{$small-screen})";
// $medium: "only screen and (min-width: #{$medium-screen})";
// $large: "only screen and (min-width: #{$large-screen})";
// $landscape: "only screen and (orientation: landscape)";
// $portrait: "only screen and (orientation: portrait)";

//// Cursors

//Custom use example -> $cursor-default-value: url(http://cursors-site.net/path/to/custom/cursor/default.cur),progress;

// $cursor-crosshair-value: "crosshair";
// $cursor-default-value: "default";
// $cursor-pointer-value: "pointer";
// $cursor-help-value: "help";

//
// Grid Variables
//

$row-width: emCalc(1140);
$column-gutter: emCalc(24);
// $total-columns: 12;

//
// Block Grid Variables
//

// We use this to control the maximum number of block grid elements per row

// $block-grid-elements: 12;
// $block-grid-default-spacing: emCalc(20);

// Enables media queries for block-grid classes. Set to false if writing semantic HTML.

// $block-grid-media-queries: true;

//
// Typography Variables
//

// Control header font styles

$header-font-family: inherit;
$header-font-weight: 300;
$header-font-style: normal;
// $header-font-color: #222;
// $header-line-height: 1.4;
// $header-top-margin: .2em;
// $header-bottom-margin: .5em;
// $header-text-rendering: optimizeLegibility;

// Control header font sizes

$h1-font-size: emCalc(36);
$h2-font-size: emCalc(24);
$h3-font-size: emCalc(21);
$h4-font-size: emCalc(18);
$h5-font-size: 1em;
$h6-font-size: .875em;

// Control how subheaders are styled.

// $subheader-line-height: 1.4;
// $subheader-font-color: lighten($header-font-color, 30%);
// $subheader-font-weight: 300;
// $subheader-top-margin: .2em;
// $subheader-bottom-margin: .5em;

// A general <small> styling

// $small-font-size: 60%;
// $small-font-color: lighten($header-font-color, 30%);

// Style paragraphs

// $paragraph-font-family: inherit;
// $paragraph-font-weight: normal;
// $paragraph-font-size: 1em;
// $paragraph-line-height: 1.6;
$paragraph-margin-bottom: emCalc(16);
// $paragraph-aside-font-size: emCalc(14);
// $paragraph-aside-line-height: 1.35;
// $paragraph-aside-font-style: italic;
// $paragraph-text-rendering: optimizeLegibility;

// Style <code> tags

// $code-color: darken($alert-color, 15%);
// $code-font-family: Consolas, 'Liberation Mono', Courier, monospace;
// $code-font-weight: bold;

// Style anchors

// $anchor-text-decoration: none;
// $anchor-font-color: $primary-color;
$anchor-font-color-hover: darken($primary-color, 10%);

// Style the <hr> element

// $hr-border-width: 1px;
// $hr-border-style: solid;
// $hr-border-color: #ddd;
// $hr-margin: emCalc(20);

// Style lists

// $list-style-position: outside;
// $list-side-margin: 0;
// $list-nested-margin: emCalc(20);
// $definition-list-header-weight: bold;
// $definition-list-header-margin-bottom: .3em;
// $definition-list-margin-bottom: emCalc(12);

// Style blockquotes

// $blockquote-font-color: lighten($header-font-color, 30%);
// $blockquote-padding: emCalc(9, 20, 0, 19);
// $blockquote-border: 1px solid #ddd;
// $blockquote-cite-font-size: emCalc(13);
// $blockquote-cite-font-color: lighten($header-font-color, 20%);
// $blockquote-cite-link-color: $blockquote-cite-font-color;

// Acronym styles

// $acronym-underline: 1px dotted #ddd;

// Control padding and margin

// $microformat-padding: emCalc(10, 12);
// $microformat-margin: emCalc(0, 0, 20, 0);

// Control the border styles

// $microformat-border-width: 1px;
// $microformat-border-style: solid;
// $microformat-border-color: #ddd;

// Control full name font styles

// $microformat-fullname-font-weight: bold;
// $microformat-fullname-font-size: emCalc(15);

// Control the summary font styles

// $microformat-summary-font-weight: bold;

// Control abbr padding
// $microformat-abbr-padding: emCalc(0, 1);

// Control abbr font styles

// $microformat-abbr-font-weight: bold;
// $microformat-abbr-font-decoration: none;

//
// Form Variables
//

// We use this to set the base for lots of form spacing and positioning styles

// $form-spacing: emCalc(16);

// We use these to style the labels in different ways

// $form-label-pointer: pointer;
// $form-label-font-size: emCalc(14);
// $form-label-font-weight: 500;
// $form-label-font-color: lighten(#000, 30%);
// $form-label-bottom-margin: emCalc(3);
// $input-font-family: inherit;
// $input-font-color: rgba(0,0,0,0.75);
// $input-font-size: emCalc(14);
// $input-bg-color: #fff;
// $input-focus-bg-color: darken(#fff, 2%);
// $input-border-color: darken(#fff, 20%);
// $input-focus-border-color: darken(#fff, 40%);
// $input-border-style: solid;
// $input-border-width: 1px;
// $input-disabled-bg: #ddd;
// $input-box-shadow: inset 0 1px 2px rgba(0,0,0,0.1);
// $input-include-glowing-effect: true;

// We use these to style the fieldset border and spacing.

// $fieldset-border-style: solid;
// $fieldset-border-width: 1px;
// $fieldset-border-color: #ddd;
// $fieldset-padding: emCalc(20);
// $fieldset-margin: emCalc(18, 0);

// We use these to style the legends when you use them

// $legend-bg: #fff;
// $legend-font-weight: bold;
// $legend-padding: emCalc(0, 3);

// We use these to style the prefix and postfix input elements

// $input-prefix-bg: darken(#fff, 5%);
// $input-prefix-border-color: darken(#fff, 20%);
// $input-prefix-border-size: 1px;
// $input-prefix-border-type: solid;
// $input-prefix-overflow: hidden;
// $input-prefix-font-color: #333;
// $input-prefix-font-color-alt: #fff;

// We use these to style the error states for inputs and labels

// $input-error-message-padding: emCalc(6, 4);
// $input-error-message-top: 0;
// $input-error-message-font-size: emCalc(12);
// $input-error-message-font-weight: bold;
// $input-error-message-font-color: #fff;
// $input-error-message-font-color-alt: #333;

// We use this to style the glowing effect of inputs when focused

// $glowing-effect-fade-time: 0.45s;
// $glowing-effect-color: $input-focus-border-color;

//
// Button Variables
//

// We use these to build padding for buttons.

// $button-med: emCalc(12);
// $button-tny: emCalc(7);
// $button-sml: emCalc(9);
// $button-lrg: emCalc(16);

// We use this to control the display property.

// $button-display: inline-block;
// $button-margin-bottom: emCalc(20);

// We use these to control button text styles.

// $button-font-family: inherit;
// $button-font-color: #fff;
// $button-font-color-alt: #333;
// $button-font-med: emCalc(16);
// $button-font-tny: emCalc(11);
// $button-font-sml: emCalc(13);
// $button-font-lrg: emCalc(20);
$button-font-weight: normal;
// $button-font-align: center;

// We use these to control various hover effects.

// $button-function-factor: 10%;

// We use these to control button border styles.

$button-border-width: 0;
$button-border-style: none;

// We use this to set the default radius used throughout the core.

// $button-radius: $global-radius;
// $button-round: $global-rounded;

// We use this to set default opacity for disabled buttons.

// $button-disabled-opacity: 0.6;

//
// Button Groups
//

// Sets the margin for the right side by default, and the left margin if right-to-left direction is used

// $button-bar-margin-opposite: emCalc(10);

//
// Dropdown Button Variables
//

// We use these to set the color of the pip in dropdown buttons

// $dropdown-button-pip-color: #fff;
// $dropdown-button-pip-color-alt: #333;

// We use these to style tiny dropdown buttons

// $dropdown-button-padding-tny: $button-tny * 5;
// $dropdown-button-pip-size-tny: $button-tny;
// $dropdown-button-pip-opposite-tny: $button-tny * 2;
// $dropdown-button-pip-top-tny: -$button-tny / 2 + emCalc(1);

// We use these to style small dropdown buttons

// $dropdown-button-padding-sml: $button-sml * 5;
// $dropdown-button-pip-size-sml: $button-sml;
// $dropdown-button-pip-opposite-sml: $button-sml * 2;
// $dropdown-button-pip-top-sml: -$button-sml / 2 + emCalc(1);

// We use these to style medium dropdown buttons

// $dropdown-button-padding-med: $button-med * 4 + emCalc(3);
// $dropdown-button-pip-size-med: $button-med - emCalc(3);
// $dropdown-button-pip-opposite-med: $button-med * 2;
// $dropdown-button-pip-top-med: -$button-med / 2 + emCalc(2);

// We use these to style large dropdown buttons

// $dropdown-button-padding-lrg: $button-lrg * 4;
// $dropdown-button-pip-size-lrg: $button-lrg - emCalc(6);
// $dropdown-button-pip-opposite-lrg: $button-lrg + emCalc(12);
// $dropdown-button-pip-top-lrg: -$button-lrg / 2 + emCalc(3);

//
// Split Button Variables
//

// We use these to control different shared styles for Split Buttons

// $split-button-function-factor: 15%;
// $split-button-pip-color: #fff;
// $split-button-pip-color-alt: #333;
// $split-button-active-bg-tint: rgba(0,0,0,0.1);

// We use these to control tiny split buttons

// $split-button-padding-tny: $button-tny * 9;
// $split-button-span-width-tny: $button-tny * 6.5;
// $split-button-pip-size-tny: $button-tny;
// $split-button-pip-top-tny: $button-tny * 2;
// $split-button-pip-default-float-tny: emCalc(-5);

// We use these to control small split buttons

// $split-button-padding-sml: $button-sml * 7;
// $split-button-span-width-sml: $button-sml * 5;
// $split-button-pip-size-sml: $button-sml;
// $split-button-pip-top-sml: $button-sml * 1.5;
// $split-button-pip-default-float-sml: emCalc(-9);

// We use these to control medium split buttons

// $split-button-padding-med: $button-med * 6.4;
// $split-button-span-width-med: $button-med * 4;
// $split-button-pip-size-med: $button-med - emCalc(3);
// $split-button-pip-top-med: $button-med * 1.5;
// $split-button-pip-default-float-med: emCalc(-9);

// We use these to control large split buttons

// $split-button-padding-lrg: $button-lrg * 6;
// $split-button-span-width-lrg: $button-lrg * 3.75;
// $split-button-pip-size-lrg: $button-lrg - emCalc(6);
// $split-button-pip-top-lrg: $button-lrg + emCalc(5);
// $split-button-pip-default-float-lrg: emCalc(-9);

//
// Alert Box Variables
//

// We use this to control alert padding.

// $alert-padding-top: emCalc(11);
// $alert-padding-default-float: $alert-padding-top;
// $alert-padding-opposite-direction: $alert-padding-top + emCalc(10);
// $alert-padding-bottom: $alert-padding-top + emCalc(1);

// We use these to control text style.

// $alert-font-weight: bold;
// $alert-font-size: emCalc(14);
// $alert-font-color: #fff;
// $alert-font-color-alt: darken($secondary-color, 60%);

// We use this for close hover effect.

// $alert-function-factor: 10%;

// We use these to control border styles.

// $alert-border-style: solid;
// $alert-border-width: 1px;
// $alert-border-color: darken($primary-color, $alert-function-factor);
// $alert-bottom-margin: emCalc(20);

// We use these to style the close buttons

// $alert-close-color: #333;
// $alert-close-position: emCalc(5);
// $alert-close-font-size: emCalc(22);
 $alert-close-opacity: 0.5;
// $alert-close-opacity-hover: 0.5;
// $alert-close-padding: 5px 4px 4px;

// We use this to control border radius

// $alert-radius: $global-radius;


//
// Breadcrumb Variables
//

// We use this to set the background color for the breadcrumb container.

$crumb-bg: #fff;

// We use these to set the padding around the breadcrumbs.

$crumb-padding: 0;
$crumb-side-padding: 0;

// We use these to control border styles.

// $crumb-function-factor: 10%;
$crumb-border-size: 0;
// $crumb-border-style: solid;
// $crumb-border-color: darken($crumb-bg, $crumb-function-factor);
$crumb-radius: 0;

// We use these to set various text styles for breadcrumbs.

$crumb-font-size: emCalc(13);
$crumb-font-color: #666361;
// $crumb-font-color-current: #333;
// $crumb-font-color-unavailable: #999;
$crumb-font-transform: none;
// $crumb-link-decor: underline;

// We use these to control the slash between breadcrumbs

$crumb-slash-color: #ccc7c3;
// $crumb-slash: "/";

//
// Clearing Variables
//

// We use these to set the background colors for parts of Clearing.

// $clearing-bg: #111;
// $clearing-caption-bg: $clearing-bg;
// $clearing-carousel-bg: #111;
// $clearing-img-bg: $clearing-bg;

// We use these to style the close button

// $clearing-close-color: #fff;
// $clearing-close-size: 40px;

// We use these to style the arrows

// $clearing-arrow-size: 16px;
// $clearing-arrow-color: $clearing-close-color;

// We use these to style captions

// $clearing-caption-font-color: #fff;
// $clearing-caption-padding: 10px 30px;

// We use these to make the image and carousel height and style

// $clearing-active-img-height: 75%;
// $clearing-carousel-height: 150px;
// $clearing-carousel-thumb-width: 175px;
// $clearing-carousel-thumb-active-border: 4px solid rgb(255,255,255);

//
// Custom Form Variables
//

// We use these to control the basic form styles input styles

// $custom-form-border-color: #ccc;
// $custom-form-border-size: 1px;
// $custom-form-bg: #fff;
// $custom-form-bg-disabled: #ddd;
// $custom-form-input-size: 16px;
// $custom-form-check-color: #222;
// $custom-form-check-size: 16px;
// $custom-form-radio-size: 8px;
// $custom-form-checkbox-radius: 0px;

// We use these to style the custom select form element.

// $custom-select-bg: #fff;
// $custom-select-fade-to-color: #f3f3f3;
// $custom-select-border-color: #ddd;
// $custom-select-triangle-color: #aaa;
// $custom-select-triangle-color-open: #222;
// $custom-select-height: emCalc(13) + ($form-spacing * 1.5);
// $custom-select-margin-bottom: emCalc(20);
// $custom-select-font-color-selected: #141414;
// $custom-select-disabled-color: #888;

// We use these to control the style of the custom select dropdown element.

// $custom-dropdown-height: 200px;
// $custom-dropdown-bg: #fff;
// $custom-dropdown-border-color: darken(#fff, 20%);
// $custom-dropdown-border-width: 1px;
// $custom-dropdown-border-style: solid;
// $custom-dropdown-font-color: #555;
// $custom-dropdown-font-size: emCalc(14);
// $custom-dropdown-color-selected: #eeeeee;
// $custom-dropdown-font-color-selected: #000;
// $custom-dropdown-shadow: 0 2px 2px 0px rgba(0,0,0,0.1);
// $custom-dropdown-offset-top: auto;
// $custom-dropdown-list-padding: emCalc(4);
// $custom-dropdown-default-float-padding: emCalc(6);
// $custom-dropdown-opposite-padding: emCalc(38);
// $custom-dropdown-list-item-min-height: emCalc(24);
// $custom-dropdown-width-small: 134px;
// $custom-dropdown-width-medium: 254px;
// $custom-dropdown-width-large: 434px;

//
// Dropdown Variables
//

// We use these to controls height and width styles.

// $f-dropdown-max-width: 200px;
// $f-dropdown-height: auto;
// $f-dropdown-max-height: none;
// $f-dropdown-margin-top: 2px;

// We use this to control the background color

// $f-dropdown-bg: #fff;

// We use this to set the border styles for dropdowns.

// $f-dropdown-border-style: solid;
// $f-dropdown-border-width: 1px;
// $f-dropdown-border-color: darken(#fff, 20%);

// We use these to style the triangle pip.

// $f-dropdown-triangle-size: 6px;
// $f-dropdown-triangle-color: #fff;
// $f-dropdown-triangle-side-offset: 10px;

// We use these to control styles for the list elements.

// $f-dropdown-list-style: none;
// $f-dropdown-font-color: #555;
// $f-dropdown-font-size: emCalc(14);
// $f-dropdown-list-padding: emCalc(5, 10);
// $f-dropdown-line-height: emCalc(18);
// $f-dropdown-list-hover-bg: #eeeeee;
// $dropdown-mobile-default-float: 0;

// We use this to control the styles for when the dropdown has custom content.

// $f-dropdown-content-padding: emCalc(20);

//
// Flex Video Variables
//

// We use these to control video container padding and margins

// $flex-video-padding-top: emCalc(25);
// $flex-video-padding-bottom: 67.5%;
// $flex-video-margin-bottom: emCalc(16);

// We use this to control widescreen bottom padding

// $flex-video-widescreen-padding-bottom: 57.25%;

//
// Inline List Variables
//

// We use this to control the margins and padding of the inline list.

// $inline-list-top-margin: 0;
// $inline-list-opposite-margin: 0;
// $inline-list-bottom-margin: emCalc(17);
// $inline-list-default-float-margin: emCalc(-22);

// $inline-list-padding: 0;

// We use this to control the overflow of the inline list.

// $inline-list-overflow: hidden;

// We use this to control the list items

// $inline-list-display: block;

// We use this to control any elments within list items

// $inline-list-children-display: block;

//
// Joyride Variables
//

// Controlling default Joyride styles

// $joyride-tip-bg: rgb(0,0,0);
// $joyride-tip-default-width: 300px;
// $joyride-tip-padding: emCalc(18, 20, 24);
// $joyride-tip-border: solid 1px #555;
// $joyride-tip-radius: 4px;
// $joyride-tip-position-offset: 22px;

// Here, we're setting the tip dont styles

// $joyride-tip-font-color: #fff;
// $joyride-tip-font-size: emCalc(14);
// $joyride-tip-header-weight: bold;

// This changes the nub size

// $joyride-tip-nub-size: 14px;

// This adjusts the styles for the timer when its enabled

// $joyride-tip-timer-width: 50px;
// $joyride-tip-timer-height: 3px;
// $joyride-tip-timer-color: #666;

// This changes up the styles for the close button

// $joyride-tip-close-color: #777;
// $joyride-tip-close-size: 30px;
// $joyride-tip-close-weight: normal;

// When Joyride is filling the screen, we use this style for the bg

// $joyride-screenfill: rgba(0,0,0,0.5);

//
// Keystroke Variables
//

// We use these to control text styles.

// $keystroke-font: "Consolas", "Menlo", "Courier", monospace;
// $keystroke-font-size: emCalc(14);
// $keystroke-font-color: #222;
// $keystroke-font-color-alt: #fff;
// $keystroke-function-factor: 7%;

// We use this to control keystroke padding.

// $keystroke-padding: emCalc(2, 4, 0);

// We use these to control background and border styles.

// $keystroke-bg: darken(#fff, $keystroke-function-factor);
// $keystroke-border-style: solid;
// $keystroke-border-width: 1px;
// $keystroke-border-color: darken($keystroke-bg, $keystroke-function-factor);
// $keystroke-radius: $global-radius;

//
// Label Variables
//

// We use these to style the labels

// $label-padding: emCalc(3, 10, 4);
// $label-radius: $global-radius;

// We use these to style the label text

// $label-font-sizing: emCalc(14);
// $label-font-weight: bold;
// $label-font-color: #333;
// $label-font-color-alt: #fff;

//
// Magellan Variables
//

// $magellan-bg: #fff;
// $magellan-padding: 10px;

//
// Orbit Settings
//

// We use these to control the caption styles

// $orbit-container-bg: #f5f5f5;
// $orbit-caption-bg-old: rgb(0,0,0);
// $orbit-caption-bg: rgba(0,0,0,0.6);
// $orbit-caption-font-color: #fff;

// We use these to control the left/right nav styles

// $orbit-nav-bg-old: rgb(0,0,0);
// $orbit-nav-bg: rgba(0,0,0,0.6);

// We use these to control the timer styles

// $orbit-timer-bg-old: rgb(0,0,0);
// $orbit-timer-bg: rgba(0,0,0,0.6);

// We use these to control the bullet nav styles

// $orbit-bullet-nav-color: #999;
// $orbit-bullet-nav-color-active: #555;

// We use thes to controls the style of slide numbers

// $orbit-slide-number-bg: rgba(0,0,0,0);
// $orbit-slide-number-font-color: #fff;
// $orbit-slide-number-padding: emCalc(5px);

// Graceful Loading Wrapper and preloader

// $wrapper-class: "slideshow-wrapper";
// $preloader-class: "preloader";

//
// Pagination Variables
//

// We use these to control the pagination container

// $pagination-height: emCalc(24);
// $pagination-margin: emCalc(-5);

// We use these to set the list-item properties

// $pagination-li-float: $default-float;
// $pagination-li-height: emCalc(24);
// $pagination-li-font-color: #222;
// $pagination-li-font-size: emCalc(14);
// $pagination-li-margin: emCalc(5);

// We use these for the pagination anchor links

// $pagination-link-pad: emCalc(1, 7, 1);
// $pagination-link-font-color: #999;
// $pagination-link-active-bg: darken(#fff, 10%);

// We use these for disabled anchor links

// $pagination-link-unavailable-cursor: default;
// $pagination-link-unavailable-font-color: #999;
// $pagination-link-unavailable-bg-active: transparent;

// We use these for currently selected anchor links

// $pagination-link-current-background: $primary-color;
// $pagination-link-current-font-color: #fff;
// $pagination-link-current-font-weight: bold;
// $pagination-link-current-cursor: default;
// $pagination-link-current-active-bg: $primary-color;

//
// Panel Variables
//

// We use these to control the background and border styles

// $panel-bg: darken(#fff, 5%);
// $panel-border-style: solid;
// $panel-border-size: 1px;

// We use this % to control how much we darken things on hover

// $panel-function-factor: 10%;
// $panel-border-color: darken($panel-bg, $panel-function-factor);

// We use these to set default inner padding and bottom margin

// $panel-margin-bottom: emCalc(20);
// $panel-padding: emCalc(20);

// We use these to set default font colors

// $panel-font-color: #333;
// $panel-font-color-alt: #fff;

// $panel-header-adjust: true;

//
// Pricing Table Variables
//

// We use this to control the border color

// $price-table-border: solid 1px #ddd;

// We use this to control the bottom margin of the pricing table

// $price-table-margin-bottom: emCalc(20);

// We use these to control the title styles

// $price-title-bg: #ddd;
// $price-title-padding: emCalc(15, 20);
// $price-title-align: center;
// $price-title-color: #333;
// $price-title-weight: bold;
// $price-title-size: emCalc(16);

// We use these to control the price styles

// $price-money-bg: #eee;
// $price-money-padding: emCalc(15, 20);
// $price-money-align: center;
// $price-money-color: #333;
// $price-money-weight: normal;
// $price-money-size: emCalc(20);

// We use these to control the description styles

// $price-bg: #fff;
// $price-desc-color: #777;
// $price-desc-padding: emCalc(15);
// $price-desc-align: center;
// $price-desc-font-size: emCalc(12);
// $price-desc-weight: normal;
// $price-desc-line-height: 1.4;
// $price-desc-bottom-border: dotted 1px #ddd;

// We use these to control the list item styles

// $price-item-color: #333;
// $price-item-padding: emCalc(15);
// $price-item-align: center;
// $price-item-font-size: emCalc(14);
// $price-item-weight: normal;
// $price-item-bottom-border: dotted 1px #ddd;

// We use these to control the CTA area styles

// $price-cta-bg: #f5f5f5;
// $price-cta-align: center;
// $price-cta-padding: emCalc(20, 20, 0);

//
// Progress Bar Variables
//

// We use this to se the prog bar height

// $progress-bar-height: emCalc(25);
// $progress-bar-color: transparent;

// We use these to control the border styles

// $progress-bar-border-color: darken(#fff, 20%);
// $progress-bar-border-size: 1px;
// $progress-bar-border-style: solid;
// $progress-bar-border-radius: $global-radius;

// We use these to control the margin & padding

// $progress-bar-pad: emCalc(2);
// $progress-bar-margin-bottom: emCalc(10);

// We use these to set the meter colors

// $progress-meter-color: $primary-color;
// $progress-meter-secondary-color: $secondary-color;
// $progress-meter-success-color: $success-color;
// $progress-meter-alert-color: $alert-color;

//
// Reveal Variables
//

// We use these to control the style of the reveal overlay.

// $reveal-overlay-bg: rgba(#000, .45);
// $reveal-overlay-bg-old: #000;

// We use these to control the style of the modal itself.

// $reveal-modal-bg: #fff;
// $reveal-position-top: 50px;
// $reveal-default-width: 80%;
// $reveal-modal-padding: emCalc(20);
// $reveal-box-shadow: 0 0 10px rgba(#000,.4);

// We use these to style the reveal close button

// $reveal-close-font-size: emCalc(22);
// $reveal-close-top: emCalc(8);
// $reveal-close-side: emCalc(11);
// $reveal-close-color: #aaa;
// $reveal-close-weight: bold;

// We use these to control the modal border

// $reveal-border-style: solid;
// $reveal-border-width: 1px;
// $reveal-border-color: #666;

// $reveal-modal-class: "reveal-modal";
// $close-reveal-modal-class: "close-reveal-modal";

//
// Section Variables
//

// We use these to set padding and hover factor

// $section-title-padding: emCalc(15);
// $section-content-padding: emCalc(15);
// $section-function-factor: 10%;

// These style the titles

// $section-title-color: #333;
// $section-title-color-active: #333;
// $section-title-bg: #efefef;
// $section-title-bg-active: darken($section-title-bg, $section-function-factor);
// $section-title-bg-active-tabs: #fff;
// $section-title-bg-hover: darken($section-title-bg, $section-function-factor / 2);

// Want to control border size, here ya go!

// $section-border-size: 1px;
// $section-border-style: solid;
// $section-border-color: #ccc;

// Font controls

// $section-font-size: emCalc(14);

// Control the color of the background and some size options

// $section-content-bg: #fff;
// $section-vertical-nav-min-width: emCalc(200);
// $section-vertical-tabs-title-width: emCalc(200);
// $section-bottom-margin: emCalc(20);

// $title-selector: ".title";
// $content-selector: ".content";
// $active-region-selector: ".active";

//
// Side Nav Variables
//

// We use this to control padding.

// $side-nav-padding: emCalc(14, 0);

// We use these to control list styles.

// $side-nav-list-type: none;
// $side-nav-list-position: inside;
// $side-nav-list-margin: emCalc(0, 0, 7, 0);

// We use these to control link styles.

// $side-nav-link-color: $primary-color;
// $side-nav-link-color-active: lighten(#000, 30%);
// $side-nav-font-size: emCalc(14);
// $side-nav-font-weight: bold;

// We use these to control border styles

// $side-nav-divider-size: 1px;
// $side-nav-divider-style: solid;
// $side-nav-divider-color: darken(#fff, 10%);

//
// Sub Nav Variables
//

// We use these to control margin and padding

// $sub-nav-list-margin: emCalc(-4, 0, 18);
// $sub-nav-list-padding-top: emCalc(4);

// We use this to control the definition

// $sub-nav-font-size: emCalc(14);
// $sub-nav-font-color: #999;
// $sub-nav-font-weight: normal;
// $sub-nav-text-decoration: none;
// $sub-nav-border-radius: 1000px;

// We use these to control the active item styles

// $sub-nav-active-font-weight: bold;
// $sub-nav-active-bg: $primary-color;
// $sub-nav-active-color: #fff;
// $sub-nav-active-padding: emCalc(3, 9);
// $sub-nav-active-cursor: default;

//
// Switch Variables
//

// Controlling border styles and background colors for the switch container

// $switch-border-color: darken(#fff, 20%);
// $switch-border-style: solid;
// $switch-border-width: 1px;
// $switch-bg: #fff;

// We use these to control the switch heights for our default classes

// $switch-height-tny: 22px;
// $switch-height-sml: 28px;
// $switch-height-med: 36px;
// $switch-height-lrg: 44px;
// $switch-bottom-margin: emCalc(20);

// We use these to control default font sizes for our classes.

// $switch-font-size-tny: 11px;
// $switch-font-size-sml: 12px;
// $switch-font-size-med: 14px;
// $switch-font-size-lrg: 17px;
// $switch-label-side-padding: 6px;

// We use these to style the switch-paddle

// $switch-paddle-bg: #fff;
// $switch-paddle-fade-to-color: darken($switch-paddle-bg, 10%);
// $switch-paddle-border-color: darken($switch-paddle-bg, 35%);
// $switch-paddle-border-width: 1px;
// $switch-paddle-border-style: solid;
// $switch-paddle-transition-speed: .1s;
// $switch-paddle-transition-ease: ease-out;
// $switch-positive-color: lighten($success-color, 50%);
// $switch-negative-color: #f5f5f5;

// Outline Style for tabbing through switches

// $switch-label-outline: 1px dotted #888;

//
// Table Variables
//

// These control the background color for the table and even rows

// $table-bg: #fff;
// $table-even-row-bg: #f9f9f9;

// These control the table cell border style

// $table-border-style: solid;
// $table-border-size: 1px;
// $table-border-color: #ddd;

// These control the table head styles

// $table-head-bg: #f5f5f5;
// $table-head-font-size: emCalc(14);
// $table-head-font-color: #222;
// $table-head-font-weight: bold;
// $table-head-padding: emCalc(8, 10, 10);

// These control the row padding and font styles

// $table-row-padding: emCalc(9, 10);
// $table-row-font-size: emCalc(14);
// $table-row-font-color: #222;
// $table-line-height: emCalc(18);

// These are for controlling the display and margin of tables

// $table-display: table-cell;
// $table-margin-bottom: emCalc(20);

//
// Image Thumbnail Variables
//

// We use these to control border styles

// $thumb-border-style: solid;
// $thumb-border-width: 4px;
// $thumb-border-color: #fff;
// $thumb-box-shadow: 0 0 0 1px rgba(#000,.2);
// $thumb-box-shadow-hover: 0 0 6px 1px rgba($primary-color,0.5);

// Radius and transition speed for thumbs

// $thumb-radius: $global-radius;
// $thumb-transition-speed: 200ms;

//
// Tooltip Variables
//

// $has-tip-border-bottom: dotted 1px #ccc;
// $has-tip-font-weight: bold;
// $has-tip-font-color: #333;
// $has-tip-border-bottom-hover: dotted 1px darken($primary-color, 20%);
// $has-tip-font-color-hover: $primary-color;
// $has-tip-cursor-type: help;

// $tooltip-padding: emCalc(8);
// $tooltip-bg: #000;
// $tooltip-font-size: emCalc(15);
// $tooltip-font-weight: bold;
// $tooltip-font-color: #fff;
// $tooltip-line-height: 1.3;
// $tooltip-close-font-size: emCalc(10);
// $tooltip-close-font-weight: normal;
// $tooltip-close-font-color: #888;
// $tooltip-font-size-sml: emCalc(14);
// $tooltip-radius: $global-radius;
// $tooltip-pip-size: 5px;

//
// Top Bar Variables
//

// Background color for the top bar

//$topbar-bg: #fff;

// Height and margin

// $topbar-height: 1em;
// $topbar-margin-bottom: 0;

// Control Input height for top bar

// $topbar-input-height: 2.45em;

// Controlling the styles for the title in the top bar

// $topbar-title-weight: bold;
// $topbar-title-font-size: emCalc(17);

// Style the top bar dropdown elements

// $topbar-dropdown-bg: #222;
// $topbar-dropdown-link-color: $colour_dark_grey;
// $topbar-dropdown-link-bg: lighten($topbar-bg, 5%);
// $topbar-dropdown-toggle-size: 5px;
// $topbar-dropdown-toggle-color: #fff;
// $topbar-dropdown-toggle-alpha: 0.5;

// Set the link colors and styles for top-level nav

// $topbar-link-color: #666361;
// $topbar-link-color-hover: #222;
// $topbar-link-color-active: #222;
// $topbar-link-weight: 700;
// $topbar-link-font-size: emCalc(15);
// $topbar-link-hover-lightness: -30%; // Darken by 30%
// $topbar-link-bg-hover: darken($topbar-bg, 3%);
// $topbar-link-bg-active: transparent;

// $topbar-dropdown-label-color: #555;
// $topbar-dropdown-label-text-transform: uppercase;
// $topbar-dropdown-label-font-weight: bold;
// $topbar-dropdown-label-font-size: emCalc(10);

// Top menu icon styles

// $topbar-menu-link-transform: uppercase;
// $topbar-menu-link-font-size: emCalc(13);
// $topbar-menu-link-weight: bold;
// $topbar-menu-link-color: #fff;
// $topbar-menu-icon-color: #fff;
// $topbar-menu-link-color-toggled: #888;
// $topbar-menu-icon-color-toggled: #888;

// Transitions and breakpoint styles

// $topbar-transition-speed: 300ms;
// $topbar-breakpoint: emCalc(940); // Change to 9999px for always mobile layout
// $topbar-media-query: "only screen and (min-width: #{$topbar-breakpoint})";

// Divider Styles

// $topbar-divider-border-bottom: solid 1px lighten($topbar-bg, 10%);
// $topbar-divider-border-top: solid 1px darken($topbar-bg, 10%);

// Sticky Class

// $topbar-sticky-class: ".sticky";


